/******************************************
 *       UNIQUE SEQUENCE IDENTIFIER       *
 *              Written By:               *
 *              Ross Walker               *
 *           Dept of Chemistry            *
 *            Imperial College            *
 *                 London                 *
 *                SW7 2AY                 *
 ******************************************/

/* This program will scan a protein seuqence
   of the form ALA,ASP,GLU,ALA,MET,ASP etc from
   a file defined by the command line switch
   -i filename.in
   and will search for the SHORTEST unique
   sequence that exists in the protein.
*/

/*There are a number of different alogorithms for
  doing this search and the options available can be
  specified on the command line with the option
  -a x, where x is a number representing the algorithm
  option.

  Current Alogritms available are:

  1 - Searches through the sequence in linear order starting with
      a search residue of length 1, checks all of the 20 common
      amino acids, if more than 1 match is found in each case it
      then increases the search string to 2 amino acids and tries
      all combinations of this, then tries length 3 etc until only
      1 match is found.
      
      Fortunately by employing loop skipout routines the combinations
      are reduced since as soon as more than one match is found it
      skips the rest.

      Note: This method requires very little memory, simply the
            length of the protein sequence, the search sting and
            a few variables - however, it is very processor intensive
            with the combinations needing to be checked scaling as
            20^n where n is the length of the sequence required to get
            a unique match
   
   2 - Pre-scans the entire sequence to check if any of the 20 amino
       acids supported are not present. If they are not present then
       it scans as above but ignores all cobinations that contain
       an unused amino acid.
       
       Note: This method is more efficient than method 1 for situations
             where only a select group of the 20 common amino acids is
             present.
             
             It needs sufficient memory to store the unused list but other
             than that no more than method 1
*/

/*See definitions.h for compile time options*/

#include <stdio.h>
#include <time.h>
#include <malloc.h>
#include <string.h>

#include "definitions.h"

char *sequence_filename;
int algorithm;
int *seq_data;
intlong seq_data_length=0;
intlong start_search=1;

int main(int argc, char *argv[])
{
  int i;
  
  /*variables used for elapsed time*/
  time_t starttime;
  time_t endtime;
  clock_t prog_start_time;
  clock_t prog_end_time;  

  starttime=time(NULL); /*Wall clock time*/
  prog_start_time=clock(); /*CPU Time*/

 /*Step 1, print program info*/
  print_prog_info();

 /*set defaults*/
 algorithm=1;
 sequence_filename=malloc(13*sizeof(char));
 if (sequence_filename==NULL)
 {
 	malloc_failure_char("main", "sequence_filename", (13*sizeof(char)));
 	return 1;
 }
 strcpy(sequence_filename,"sequence.dat");
 /*End set default options*/

  i=process_command_line(argc, argv);    /*Return values are
                                                   0 = success
                                                   1 = failure unknown option
                                                   2 = failure too many options
                                                   3 = allocation failure
                                                   4 = Help requested
                                                   5 = Program History Requested */
  if (i == 1){
      printf("\n*** UNKNOWN COMMAND LINE OPTION ***\n");
      command_line_help(argv);
      global_unlock();
      return 1;
  }
  if (i == 2){
      printf("\n*** TOO MANY COMMAND LINE OPTIONS ***\n");
      command_line_help(argv);
      global_unlock();
      return 1;
  }
  if (i == 3){
      printf("\n*** FATAL ERROR IN process_command_line(argc, argv) ***\n");
      printf("\n*** MEMORY ALLOCATION FAILED ***\n");
      printf("\n*** ABNORMAL PROGRAM TERMINATION ***\n");
      global_unlock();
      return 1;
  }
  if (i == 4){
    	/*The user requested help so we print out the help options*/
       command_line_help(argv);     /*Prints out the command line options */
       global_unlock();
       return 0; /*Nothing else to do so we quit*/
  }
  if (i == 5){
    	/*The user requested program history so we print out the history*/
       program_history();     /*Prints out the programs development history */
       global_unlock();
       return 0; /*Nothing else to do so we quit*/
  }

 
  printf("*************************************************************\n");
  printf("\n  Program Execution Started at %s\n",asctime(localtime(&starttime)));
 
  if (algorithm==1)
  {
  	i=run_algorithm_1();
  	if (i!=0)
  		return 1; /*FAILURE*/
  }
  else if (algorithm==2)
  {
    i=run_algorithm_2();
 	if (i!=0)
  		return 1; /*FAILURE*/
  }
  else
  {
  	printf("\nUNKNOWN ALGORITHM\n");
  } 
 
  
  endtime = time(NULL);
  prog_end_time=clock(); /*CPU Time*/  
  printf("\n\n  Program Execution Completed at %s\n",asctime(localtime(&endtime)));
  printf("                   Elapsed Time = %.1f seconds\n",difftime(endtime,starttime));
  printf("*************************************************************\n");

  return 0;
}

